/*
 There are two possible ways to work with the library Gipals32.dll.

 1) First way is a linear program construction by the provided set of functions,
    such as specification of the variables and constraints count and assignment
    of the constraint matrix elements. This way provides a complete flexibility
    of the linear programming modeling.

 2) The second way is importing linear program from a text file in MPS format.
    It can be done by one function LoadFromMPS. This way can be suitable if there
    is a set of prepared MPS files that need to be calculated.

  The example program work as following:
  If there is a command line parameter then it's considered as MPS file and the
  program tries to load and calculate it.
  If there are no command line parameters then internal example of a transport
  problem is loaded and calculated.
*/

#pragma comment(lib, "gipals32.lib"); 
#include "stdafx.h"
#include "HeaderGipalsVC.h"


int IterNo;

int __stdcall ProcessInfo (int IterCount,     // Current iteration
                                  double PrimalObj,  // Primal objective function
                                  double DualObj,    // Dual objective function
                                  double InfPrimal,  // Primal infeasibility
                                  double InfDual,    // Dual infeasibility 
                                  double Optimality) // Optimality 
{
  if (IterCount != IterNo)
  {
	  IterNo = IterCount;
      printf("%5d | %12g | %12g | %12g | %12g | %12g \n", 
	         IterCount, 
		     PrimalObj,
			 DualObj,
			 InfPrimal,
			 InfDual,
			 Optimality);

  }
  return res_None;
}

/*
  This is an example of a transport problem (Minimal Cost).

  There are three warehouses at different cities: Detroit, Pittsburgh and Buffalo.
  They have 250, 130 and 235 tons of paper accordingly. There are four publishers in
  Boston, New York, Chicago and Indianapolis. They ordered 75, 230, 240 and 70 tons of
  paper to publish new books.
  There are the following costs in dollars of transportation of one ton of paper:

  From \ To	Boston (BS)	New York (NY)	Chicago (CH)	Indianapolis (IN)
  Detroit (DT)	         15                20             16                   21
  Pittsburgh (PT)        25                13              5                   11
  Buffalo (BF)           15                15              7                   17

  We denote the cost of transportation from one city to another by C_ prefix, for example
  cost from Buffalo to Chicago is C_BF_CH.

  We have to find a plan that all orders will be performed and the transportation costs
  will be minimized. The decision variables are amounts of tons transported from one
  city to another denoted by prefix X_:

  X_DT_BS = tons from Detroit to Boston,
  X_PT_BS = tons from Pittsburgh to Boston,
  X_BF_NY = tons from Buffalo to New York and etc.

  The objective function is sum of multiplications of transported tons and their costs:
  F = C_DT_BS * X_DT_BS + C_DT_NY * C_DT_NY + C_DT_CH * C_DT_CH + 

  The constraints are defined in terms of the transported paper amount from one
  warehouse or to one publisher.

  For example:
  Amount transported from Detroit cannot be more than 250 tons:
  X_DT_BS + X_DT_NY + X_DT_CH + X_DT_IN <= 250
*/

void FillVariables()
{
  AddVariable(15, 0, 0, 0, 1); // Tons from Detroit to Boston. Cost per tone is $15
  AddVariable(20, 0, 0, 0, 1); // Tons from Detroit to New York. Cost per tone is $20
  AddVariable(16, 0, 0, 0, 1); // Tons from Detroit to Chicago. Cost per tone is $16
  AddVariable(21, 0, 0, 0, 1); // Tons from Detroit to Indianapolis. Cost per tone is $21
  AddVariable(25, 0, 0, 0, 1); // Tons from Pittsburgh to Boston. Cost per tone is $25
  AddVariable(13, 0, 0, 0, 1); // Tons from Pittsburgh to New York. Cost per tone is $13
  AddVariable( 5, 0, 0, 0, 1); // Tons from Pittsburgh to Chicago. Cost per tone is $5
  AddVariable(11, 0, 0, 0, 1); // Tons from Pittsburgh to Indianapolis. Cost per tone is $11
  AddVariable(15, 0, 0, 0, 1); // Tons from Buffalo to Boston. Cost per tone is $15
  AddVariable(15, 0, 0, 0, 1); // Tons from Buffalo to New York. Cost per tone is $15
  AddVariable( 7, 0, 0, 0, 1); // Tons from Buffalo to Chicago. Cost per tone is $7
  AddVariable(17, 0, 0, 0, 1); // Tons from Buffalo to Indianapolis.Cost per tone is $17
};

void FillConstraints()
{
  AddConstraint(250, sign_Less);  // Transported from Detroit must be less then 250 tones.
  AddConstraint(130, sign_Less);  // Transported from Pittsburgh must be less then 130.
  AddConstraint(235, sign_Less);  // Transported from Buffalo must be less then 235.
  AddConstraint( 75, sign_Equal); // Shipped to Boston must be 75 tones.
  AddConstraint(230, sign_Equal); // Shipped to New York must be 230 tones.
  AddConstraint(240, sign_Equal); // Shipped to Chicago must be 240 tones.
  AddConstraint( 70, sign_Equal); // Shipped to Indianapolis must be 70 tones.
};

void FillMatrix()
{
  // X_DT_BS + X_DT_NY + X_DT_CH + X_DT_IN <= 250
  DirectSelectConstraint(0);
  DirectAddConstraintElement(0, 1); // X_DT_BS
  DirectAddConstraintElement(1, 1); // X_DT_NY
  DirectAddConstraintElement(2, 1); // X_DT_CH
  DirectAddConstraintElement(3, 1); // X_DT_IN

  // X_PT_BS + X_PT_NY + X_PT_CH + X_PT_IN <= 130
  DirectSelectConstraint(1);
  DirectAddConstraintElement(4, 1); // X_PT_BS
  DirectAddConstraintElement(5, 1); // X_PT_NY
  DirectAddConstraintElement(6, 1); // X_PT_CH
  DirectAddConstraintElement(7, 1); // X_PT_IN

  // X_BF_BS + X_BF_NY + X_BF_CH + X_BF_IN <= 235
  DirectSelectConstraint(2);
  DirectAddConstraintElement(8, 1); // X_BF_BS
  DirectAddConstraintElement(9, 1); // X_BF_NY
  DirectAddConstraintElement(10, 1); // X_BF_CH
  DirectAddConstraintElement(11, 1); // X_BF_IN

  // X_DT_BS + X_PT_BS + X_BF_BS = 75
  DirectSelectConstraint(3);
  DirectAddConstraintElement(0, 1); // X_DT_BS
  DirectAddConstraintElement(4, 1); // X_PT_BS
  DirectAddConstraintElement(8, 1); // X_BF_BS

  // X_DT_NY + X_PT_NY + X_BF_NY = 230
  DirectSelectConstraint(4);
  DirectAddConstraintElement(1, 1); // X_DT_NY
  DirectAddConstraintElement(5, 1); // X_PT_NY
  DirectAddConstraintElement(9, 1); // X_BF_NY

  // X_DT_CH + X_PT_CH + X_BF_CH = 240
  DirectSelectConstraint(5);
  DirectAddConstraintElement( 2, 1); // X_DT_CH
  DirectAddConstraintElement( 6, 1); // X_PT_CH
  DirectAddConstraintElement(10, 1); // X_BF_CH

  // X_DT_IN + X_PT_IN + X_BF_IN = 70
  DirectSelectConstraint(6);
  DirectAddConstraintElement( 3, 1); // X_DT_IN
  DirectAddConstraintElement( 7, 1); // X_PT_IN
  DirectAddConstraintElement(11, 1); // X_BF_IN
};


int main(int argc, char* argv[])
{
	
	int CalcRes;
	bool IsTransport; // Indicates transport problem

	IsTransport = (argc == 1);
    if (!IsTransport) 
	{
		printf("Importing linear program from file: %s\n", argv[1]);
		LoadFromMPS(argv[1]);
        printf("Import is done\n");
	}
	else 
    {
      printf("This is an example of a transport problem (Minimal Cost)\n");
      FillVariables();
      FillConstraints();
      FillMatrix(); 
	}

    // Print the table header
    printf("---------------------------------------------------------------------------------\n");
    printf(" No.  |    Objective Functions      |        Infeasibility        |  Optimality \n");
    printf(" Iter.|    Primal    |     Dual     |    Primal    |      Dual    |             \n");
    printf("---------------------------------------------------------------------------------\n");
    
	// Call the calculation
	CalcRes = Calculate(ProcessInfo);
	
	// Analyze the result 
	switch (CalcRes) 
	{ 
    case          calc_None: printf("Calculation has not been provided yet"); break;
    case       calc_Optimal: printf("Finished with optimal solution"); break;
    case    calc_Infeasible: printf("Failed with primal infeasibility"); break;
    case calc_DblInfeasible: printf("Failed with dual infeasibility"); break;
    case       calc_Unknown: printf("Finished because reached the max. of iterations"); break;
    case    calc_Suboptimal: printf("Finished due to slow convergence"); break;
    case        calc_Stoped: printf("Stoped by the user"); break;
    case         calc_Error: printf("Failed due to numerical eror"); break;
    case     calc_Unbounded: printf("Failed because the linear program is unbounded"); break;
    default: ;
 	}
  	printf("\n");

    if (IsTransport) 
	{
      printf("\nThe Total Cost of Transportation is $%.2f\n\n", PrimalObjective());
      // output results of the calculation
      printf("Shipment plan:\n\n");
      printf("%10.2f Tons from Detroit to Boston\n", PrimalValue(0));
      printf("%10.2f Tons from Detroit to New York\n", PrimalValue(1));
      printf("%10.2f Tons from Detroit to Chicago\n", PrimalValue(2));
      printf("%10.2f Tons from Detroit to Indianapolis\n", PrimalValue(3));
      printf("%10.2f Tons from Pittsburgh to Boston\n", PrimalValue(4));
      printf("%10.2f Tons from Pittsburgh to New York\n", PrimalValue(5));
      printf("%10.2f Tons from Pittsburgh to Chicago\n", PrimalValue(6));
      printf("%10.2f Tons from Pittsburgh to Indianapolis\n", PrimalValue(7));
      printf("%10.2f Tons from Buffalo to Boston\n", PrimalValue(8));
      printf("%10.2f Tons from Buffalo to New York\n", PrimalValue(9));
      printf("%10.2f Tons from Buffalo to Chicago\n", PrimalValue(10));
      printf("%10.2f Tons from Buffalo to Indianapolis\n", PrimalValue(11));
    }
	else
    {
	  printf("\nPrimal Objective = %f\n", PrimalObjective());
	  printf("  Dual Objective = %f\n", DualObjective());

	  // output results of the calculation
      printf("\nPrimal Results.\n");
      printf("   #    Variable (X)      Reduced Cost (S)\n");
	  for (int iter = 0; iter < GetVariableCount(); iter++)
	  {
	     printf("%4d    %12g          %12g\n", iter, PrimalValue(iter), ReducedCostValue(iter));
	  }

	  printf("\nDual Results\n");
	  printf("   #    Dual Variable (Y)\n");
	  for (iter = 0; iter < GetConstraintCount(); iter++)
	  {
	 	 printf("%4d    %12g\n", iter, DualValue(iter));
	  }
    }  
	return 0;
}

